function _slicedToArray(arr, i) { return _arrayWithHoles(arr) || _iterableToArrayLimit(arr, i) || _unsupportedIterableToArray(arr, i) || _nonIterableRest(); }

function _nonIterableRest() { throw new TypeError("Invalid attempt to destructure non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method."); }

function _unsupportedIterableToArray(o, minLen) { if (!o) return; if (typeof o === "string") return _arrayLikeToArray(o, minLen); var n = Object.prototype.toString.call(o).slice(8, -1); if (n === "Object" && o.constructor) n = o.constructor.name; if (n === "Map" || n === "Set") return Array.from(o); if (n === "Arguments" || /^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(n)) return _arrayLikeToArray(o, minLen); }

function _arrayLikeToArray(arr, len) { if (len == null || len > arr.length) len = arr.length; for (var i = 0, arr2 = new Array(len); i < len; i++) { arr2[i] = arr[i]; } return arr2; }

function _iterableToArrayLimit(arr, i) { if (typeof Symbol === "undefined" || !(Symbol.iterator in Object(arr))) return; var _arr = []; var _n = true; var _d = false; var _e = undefined; try { for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) { _arr.push(_s.value); if (i && _arr.length === i) break; } } catch (err) { _d = true; _e = err; } finally { try { if (!_n && _i["return"] != null) _i["return"](); } finally { if (_d) throw _e; } } return _arr; }

function _arrayWithHoles(arr) { if (Array.isArray(arr)) return arr; }

import { ValidationRuleTypes } from './types';
/**
 * Requires the input to be filled. Use exceptions array to define falsy values that should result to true.
 * I.E. Numeric 0 evaluates to false by default, but having `{ exceptions: [0] }` will make it so numeric 0 is accepted as
 * a valid value
 *
 * @param options RequiredRuleOptions:
 *  - **exceptions**: any[] - specifies an array of values that acts as exceptions and will validate to true when the input value matches any value
 * within the exceptions array. This is mainly used to make falsy values evaluate to true under certain cases, I.E., to evaluate numeric 0 as truthy.
 * Default `[]`
 *
 * @returns ValidationRule
 */

export var RequiredRule = function RequiredRule() {
  var _ref = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {},
      message = _ref.message,
      transformer = _ref.transformer,
      validator = _ref.validator,
      _ref$exceptions = _ref.exceptions,
      exceptions = _ref$exceptions === void 0 ? [] : _ref$exceptions;

  var verify = function verify(inputValue) {
    var input = transformer ? transformer(inputValue) : inputValue;
    return validator ? validator(input, exceptions) : !!(input || exceptions.indexOf(input) > -1);
  };

  return {
    type: ValidationRuleTypes.required,
    value: exceptions,
    message: message !== null && message !== void 0 ? message : 'Validation: RequiredRule failed',
    validator: verify
  };
};
/**
 * The input's value must match the defined regex (pattern as a string is converted to regex).
 * The inverse option makes it so the input must not match the defined pattern
 * @param pattern RegExp | string
 * @param options PatternRuleOptions:
 *  - **message**: string - Error message
 *  - **transformer**: (value: any) => any - Used to transform the input value before getting passed to validation fn
 *  - **validator**: (inputValue: any, ruleValue: any) => boolean - To override the validation fn for this rule. Default changes depending on the ValidationRule used
 *  - **inverse**: boolean - Reverses the validation result when true. Default: `false`
 *
 * @returns ValidationRule
 */

export var PatternRule = function PatternRule(pattern) {
  var _ref2 = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {},
      message = _ref2.message,
      transformer = _ref2.transformer,
      validator = _ref2.validator,
      _ref2$inverse = _ref2.inverse,
      inverse = _ref2$inverse === void 0 ? false : _ref2$inverse;

  var regex = pattern instanceof RegExp ? pattern : new RegExp(pattern);

  var verify = function verify(inputValue) {
    var input = transformer ? transformer(inputValue) : inputValue;
    var validated = validator ? validator(input, regex) : regex.test(input);
    return inverse ? !validated : validated;
  };

  return {
    type: ValidationRuleTypes.pattern,
    value: regex,
    message: message !== null && message !== void 0 ? message : 'Validation: PatternRule failed',
    validator: verify
  };
};

var defaultLengthRangeValidator = function defaultLengthRangeValidator(value, range) {
  var inputValue = typeof value === 'string' ? value.length : Number(value) || 0;

  var _range = _slicedToArray(range, 2),
      min = _range[0],
      max = _range[1];

  var passed = true;

  if (min > 0) {
    passed = inputValue >= min; //early termination

    if (!passed) {
      return passed;
    }
  }

  if (max > 0 && max > min) {
    passed = inputValue <= max;
  }

  return passed;
};
/**
 * Defines the length required for the input. Use -1 for min / max to disable min / max checks.
 * @param min number - Use -1 to ignore min length restriction. Default: `-1`
 * @param max number - Use -1 to ignore max length restriction. Default: `-1`
 * @param options ValidationRuleOptions:
 *  - **message**: string - Error message
 *  - **transformer**: (value: any) => any - Used to transform the input value before getting passed to validation fn
 *  - **validator**: (inputValue: any, ruleValue: any) => boolean - To override the validation fn for this rule. Default changes depending on the ValidationRule used
 *
 * @returns ValidationRule
 */


export var LengthRangeRule = function LengthRangeRule() {
  var min = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : -1;
  var max = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : -1;

  var _ref3 = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : {},
      message = _ref3.message,
      transformer = _ref3.transformer,
      validator = _ref3.validator;

  var range = [min, max];

  var verify = function verify(inputValue) {
    var input = transformer ? transformer(inputValue) : Number(inputValue);
    return validator ? validator(input, range) : defaultLengthRangeValidator(inputValue, range);
  };

  return {
    type: ValidationRuleTypes.lengthRange,
    value: range,
    message: message !== null && message !== void 0 ? message : 'Validation: LengthRangeRule failed',
    validator: verify
  };
};

var defaultComparator = function defaultComparator(a, b) {
  return a === b;
};
/**
 * Validate the input value against an array of values (haystack) to ensure input value's uniqueness among the array of values.
 * @param existing any[] - array of values that the input value can not be
 * @param options UniqueValueRuleOptions :
 *  - **message**: string - Error message
 *  - **transformer**: (value: any) => any - Used to transform the input value before getting passed to validation fn
 *  - **validator**: (inputValue: any, ruleValue: any) => boolean - To override the validation fn for this rule. Default changes depending on the ValidationRule used
 *  - **inverse**: boolean - reverses validation result when true. Default: `false`
 *  - **comparator**: (inputValue, haystackValue) => boolean - Used to override how input value and haystack values are compared. Defaults to using `===`
 *  - **haystackTransformer**: (haystayckValue: any) => any - Same as `transformer` but for the haystack
 *
 * @returns ValidationRule
 */


export var UniqueValueRule = function UniqueValueRule(existing) {
  var _ref4 = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {},
      message = _ref4.message,
      transformer = _ref4.transformer,
      validator = _ref4.validator,
      _ref4$inverse = _ref4.inverse,
      inverse = _ref4$inverse === void 0 ? false : _ref4$inverse,
      _ref4$comparator = _ref4.comparator,
      comparator = _ref4$comparator === void 0 ? defaultComparator : _ref4$comparator,
      haystackTransformer = _ref4.haystackTransformer;

  var verify = function verify(inputValue) {
    var input = transformer ? transformer(inputValue) : inputValue;
    var haystack = existing.map(function (exist) {
      return haystackTransformer ? haystackTransformer(exist) : exist;
    });
    var result = validator ? validator(input, haystack) : haystack.filter(function (exist) {
      return comparator(input, exist);
    }).length === 0;
    return inverse ? !result : result;
  };

  return {
    type: ValidationRuleTypes.uniqueValue,
    value: existing,
    message: message !== null && message !== void 0 ? message : 'Validation: UniqueValueRule failed',
    validator: verify
  };
};
/**
 * Custom rule. Must provide the validator function.
 * @param validator ValidatorFn - ***Required method***. Called when validation logic is triggered
 * @param options CustomRuleOptions:
 * - **message**: string - Error message
 * - **transformer**: (value: any) => any - Used to transform the input value before getting passed to validation fn
 *
 * @returns ValidationRule
 */

export var CustomRule = function CustomRule(validator) {
  var _ref5 = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {},
      message = _ref5.message,
      transformer = _ref5.transformer;

  return {
    type: ValidationRuleTypes.custom,
    value: null,
    message: message !== null && message !== void 0 ? message : 'Validation: CustomRule failed',
    validator: function (_validator) {
      function validator(_x) {
        return _validator.apply(this, arguments);
      }

      validator.toString = function () {
        return _validator.toString();
      };

      return validator;
    }(function (inputValue) {
      var value = transformer ? transformer(inputValue) : inputValue;
      return validator(value);
    })
  };
};