/// <reference types="react" />
import { ValidationInputProps } from './types';
/**
 * Higher order component that attaches the `useValidation` hook to the provided input component.
 * Default input component is set to `<input type="text" />` and binds the validation call to `onChange` event listener.
 *
 * Important props:
 * - **rules**: ValidationRule[] - an array of validation rules
 * - **InputComponent**: React.ElementType<any> | React.FC<any> - specifies what input component to use. Default: `<input />`
 * - **Wrapper**: React.ElementType<any> - specifies wrapper component around the input component. Default: `React.Fragment`
 * - **inputValueParser**: function - function used to specify how to get the input value from the input component.
 *  Pulls from `event.currentTarget.value` if undefined. Default: `undefined`
 * - **bindEventListener**: string[] - specifies which event listener to bind the validation call to and also return the validation state as 2nd argument to event handler.
 * Default: `["onChange"]`
 * - **bindValidityToEvent**: string[] - specifies which event listener to attach the validation state to (2nd argument) ***WITHOUT*** calling the validation function.
 * This has lower priority than `bindEventListener` if there is an overlap of events. Default: `[]`
 * - **validateOnMount**: boolean - validates the input on component mount when true. Default: `false`
 * - **onValidityChange**: (validity: ValidationState, inputValue: any, name?: string) => void - an event listener that triggers when validation state (`validity.passed`) changes
 * - **onValiditySuccess**: (validity: ValidationState, inputValue: any, name?: string) => void - an event listener that triggers when validation state (`validity.passed`) evaluates to true
 * - **onValidityFailure**: (validity: ValidationState, inputValue: any, name?: string) => void - an event listener that triggers when validation state (`validity.passed`) evaluates to false
 * - **children**: (validity: ValidationState, validate: ValidateFn) => React.ReactNode - Specifies additional elements to be rendered after the input component. Must be a function. Default: `undefined`.
 * The `validate` fn is to provide a manual way of calling the validation on that input component.
 *
 * The rest of the props are passed as-is into the input component
 * Note: `props.type` is defaulted to `"text"`
 * Note: `props.defaultValue` is used as the 1st input value to validate against
 *
 * @param props ValidationInputProps
 * @returns
 */
declare const ValidateInput: {
    ({ rules, defaultValue, Wrapper, InputComponent, inputValueParser, bindEventListener, bindValidityToEvent, validateOnMount, onValidityChanged, onValiditySuccess, onValidityFailure, children, ...props }: ValidationInputProps): JSX.Element;
    defaultProps: ValidationInputProps;
};
export { ValidateInput };
