import { ValidatorFn, ValidationRule, ValidationRuleOptions, RequiredRuleOptions, PatternRuleOptions, UniqueValueRuleOptions, CustomRuleOptions } from './types';
/**
 * Requires the input to be filled. Use exceptions array to define falsy values that should result to true.
 * I.E. Numeric 0 evaluates to false by default, but having `{ exceptions: [0] }` will make it so numeric 0 is accepted as
 * a valid value
 *
 * @param options RequiredRuleOptions:
 *  - **exceptions**: any[] - specifies an array of values that acts as exceptions and will validate to true when the input value matches any value
 * within the exceptions array. This is mainly used to make falsy values evaluate to true under certain cases, I.E., to evaluate numeric 0 as truthy.
 * Default `[]`
 *
 * @returns ValidationRule
 */
export declare const RequiredRule: (options?: RequiredRuleOptions) => ValidationRule;
/**
 * The input's value must match the defined regex (pattern as a string is converted to regex).
 * The inverse option makes it so the input must not match the defined pattern
 * @param pattern RegExp | string
 * @param options PatternRuleOptions:
 *  - **message**: string - Error message
 *  - **transformer**: (value: any) => any - Used to transform the input value before getting passed to validation fn
 *  - **validator**: (inputValue: any, ruleValue: any) => boolean - To override the validation fn for this rule. Default changes depending on the ValidationRule used
 *  - **inverse**: boolean - Reverses the validation result when true. Default: `false`
 *
 * @returns ValidationRule
 */
export declare const PatternRule: (pattern: RegExp | string, options?: PatternRuleOptions) => ValidationRule;
/**
 * Defines the length required for the input. Use -1 for min / max to disable min / max checks.
 * @param min number - Use -1 to ignore min length restriction. Default: `-1`
 * @param max number - Use -1 to ignore max length restriction. Default: `-1`
 * @param options ValidationRuleOptions:
 *  - **message**: string - Error message
 *  - **transformer**: (value: any) => any - Used to transform the input value before getting passed to validation fn
 *  - **validator**: (inputValue: any, ruleValue: any) => boolean - To override the validation fn for this rule. Default changes depending on the ValidationRule used
 *
 * @returns ValidationRule
 */
export declare const LengthRangeRule: (min: number, max: number, options?: ValidationRuleOptions) => ValidationRule;
/**
 * Validate the input value against an array of values (haystack) to ensure input value's uniqueness among the array of values.
 * @param existing any[] - array of values that the input value can not be
 * @param options UniqueValueRuleOptions :
 *  - **message**: string - Error message
 *  - **transformer**: (value: any) => any - Used to transform the input value before getting passed to validation fn
 *  - **validator**: (inputValue: any, ruleValue: any) => boolean - To override the validation fn for this rule. Default changes depending on the ValidationRule used
 *  - **inverse**: boolean - reverses validation result when true. Default: `false`
 *  - **comparator**: (inputValue, haystackValue) => boolean - Used to override how input value and haystack values are compared. Defaults to using `===`
 *  - **haystackTransformer**: (haystayckValue: any) => any - Same as `transformer` but for the haystack
 *
 * @returns ValidationRule
 */
export declare const UniqueValueRule: (existing: any[], options?: UniqueValueRuleOptions) => ValidationRule;
/**
 * Custom rule. Must provide the validator function.
 * @param validator ValidatorFn - ***Required method***. Called when validation logic is triggered
 * @param options CustomRuleOptions:
 * - **message**: string - Error message
 * - **transformer**: (value: any) => any - Used to transform the input value before getting passed to validation fn
 *
 * @returns ValidationRule
 */
export declare const CustomRule: (validator: ValidatorFn, options?: CustomRuleOptions) => ValidationRule;
